@@ -3,6 +3,7 @@ package client
 import (
 	"encoding/xml"
 	"fmt"
+	"runtime"
 
 	"github.com/golang/glog"
 	libvirt "github.com/libvirt/libvirt-go"
@@ -46,7 +47,7 @@ type CreateDomainInput struct {
 	HostName string
 
 	// AddressRange as IP subnet address range
-	AddressRange int
+	ReservedLeases *Leases
 
 	// Autostart as domain autostart
 	Autostart bool
@@ -108,6 +109,9 @@ type Client interface {
 	// DeleteVolume deletes a domain based on its name
 	DeleteVolume(name string) error
 
+	// GetDHCPLeasesByNetwork get all network DHCP leases by network name
+	GetDHCPLeasesByNetwork(networkName string) ([]libvirt.NetworkDHCPLease, error)
+
 	// LookupDomainHostnameByDHCPLease looks up a domain hostname based on its DHCP lease
 	LookupDomainHostnameByDHCPLease(domIPAddress string, networkName string) (string, error)
 }
@@ -174,32 +178,6 @@ func (client *libvirtClient) CreateDomain(input CreateDomainInput) error {
 		return fmt.Errorf("Failed to init domain definition from machineProviderConfig: %v", err)
 	}
 
-	glog.Info("Create ignition configuration")
-	if input.Ignition != nil {
-		if err := setIgnition(&domainDef, client, input.Ignition, input.KubeClient, input.MachineNamespace, input.IgnitionVolumeName); err != nil {
-			return err
-		}
-	} else if input.IgnKey != "" {
-		ignVolume, err := client.getVolume(input.IgnKey)
-		if err != nil {
-			return fmt.Errorf("error getting ignition volume: %v", err)
-		}
-		ignVolumePath, err := ignVolume.GetPath()
-		if err != nil {
-			return fmt.Errorf("error getting ignition volume path: %v", err)
-		}
-
-		if err := setCoreOSIgnition(&domainDef, ignVolumePath); err != nil {
-			return err
-		}
-	} else if input.CloudInit != nil {
-		if err := setCloudInit(&domainDef, client, input.CloudInit, input.KubeClient, input.MachineNamespace, input.CloudInitVolumeName, input.DomainName); err != nil {
-			return err
-		}
-	} else {
-		return fmt.Errorf("machine does not has a IgnKey nor CloudInit value")
-	}
-
 	glog.Info("Create volume")
 	diskVolume, err := client.getVolume(input.VolumeName)
 	if err != nil {
@@ -209,6 +187,42 @@ func (client *libvirtClient) CreateDomain(input CreateDomainInput) error {
 		return fmt.Errorf("Failed to setDisks: %s", err)
 	}
 
+	glog.Info("Create ignition configuration")
+
+	// Both "s390" and "s390x" are linux kernel architectures for Linux on IBM z Systems, and they are for 31-bit and 64-bit respectively.
+	if runtime.GOARCH == "s390x" || runtime.GOARCH == "s390" {
+		if input.Ignition != nil {
+			if err := setIgnitionForS390X(&domainDef, client, input.Ignition, input.KubeClient, input.MachineNamespace, input.IgnitionVolumeName); err != nil {
+				return err
+			}
+		}
+	} else {
+		if input.Ignition != nil {
+			if err := setIgnition(&domainDef, client, input.Ignition, input.KubeClient, input.MachineNamespace, input.IgnitionVolumeName); err != nil {
+				return err
+			}
+		} else if input.IgnKey != "" {
+			ignVolume, err := client.getVolume(input.IgnKey)
+			if err != nil {
+				return fmt.Errorf("error getting ignition volume: %v", err)
+			}
+			ignVolumePath, err := ignVolume.GetPath()
+			if err != nil {
+				return fmt.Errorf("error getting ignition volume path: %v", err)
+			}
+
+			if err := setCoreOSIgnition(&domainDef, ignVolumePath); err != nil {
+				return err
+			}
+		} else if input.CloudInit != nil {
+			if err := setCloudInit(&domainDef, client, input.CloudInit, input.KubeClient, input.MachineNamespace, input.CloudInitVolumeName, input.DomainName); err != nil {
+				return err
+			}
+		} else {
+			return fmt.Errorf("machine does not has a IgnKey nor CloudInit value")
+		}
+	}
+
 	glog.Info("Set up network interface")
 	var waitForLeases []*libvirtxml.DomainInterface
 	hostName := input.HostName
@@ -217,9 +231,16 @@ func (client *libvirtClient) CreateDomain(input CreateDomainInput) error {
 	}
 	// TODO: support more than 1 interface
 	partialNetIfaces := make(map[string]*pendingMapping, 1)
-	if err := setNetworkInterfaces(&domainDef, client.connection, partialNetIfaces, &waitForLeases,
-		hostName, input.NetworkInterfaceName,
-		input.NetworkInterfaceAddress, input.AddressRange); err != nil {
+	if err := setNetworkInterfaces(
+		&domainDef,
+		client.connection,
+		partialNetIfaces,
+		&waitForLeases,
+		hostName,
+		input.NetworkInterfaceName,
+		input.NetworkInterfaceAddress,
+		input.ReservedLeases,
+	); err != nil {
 		return err
 	}
 
@@ -415,7 +436,7 @@ func (client *libvirtClient) CreateVolume(input CreateVolumeInput) error {
 		// create the volume
 		// Refresh the pool of the volume so that libvirt knows it is
 		// not longer in use.
-		waitForSuccess("error refreshing pool for volume", func() error {
+		err = waitForSuccess("error refreshing pool for volume", func() error {
 			return client.pool.Refresh(0)
 		})
 		if err != nil {
@@ -526,15 +547,20 @@ func (client *libvirtClient) DeleteVolume(name string) error {
 	return nil
 }
 
-// LookupDomainHostnameByDHCPLease looks up a domain hostname based on its DHCP lease
-func (client *libvirtClient) LookupDomainHostnameByDHCPLease(domIPAddress string, networkName string) (string, error) {
+// GetDHCPLeasesByNetwork returns all network DHCP leases by network name
+func (client *libvirtClient) GetDHCPLeasesByNetwork(networkName string) ([]libvirt.NetworkDHCPLease, error) {
 	network, err := client.connection.LookupNetworkByName(networkName)
 	if err != nil {
 		glog.Errorf("Failed to fetch network %s from the libvirt", networkName)
-		return "", err
+		return nil, err
 	}
 
-	dchpLeases, err := network.GetDHCPLeases()
+	return network.GetDHCPLeases()
+}
+
+// LookupDomainHostnameByDHCPLease looks up a domain hostname based on its DHCP lease
+func (client *libvirtClient) LookupDomainHostnameByDHCPLease(domIPAddress string, networkName string) (string, error) {
+	dchpLeases, err := client.GetDHCPLeasesByNetwork(networkName)
 	if err != nil {
 		glog.Errorf("Failed to fetch dhcp leases for the network %s", networkName)
 		return "", err
